from typing import Final

BUFFER_ARRAY_BUFFER: Final[int]
BUFFER_ELEMENT_ARRAY_BUFFER: Final[int]
BUFFER_PIXEL_PACK_BUFFER: Final[int]
BUFFER_PIXEL_UNPACK_BUFFER: Final[int]
BUFFER_READ_ONLY: Final[int]
BUFFER_READ_WRITE: Final[int]
BUFFER_WRITE_ONLY: Final[int]
Buffer_ARRAY_BUFFER: Final[int]
Buffer_ELEMENT_ARRAY_BUFFER: Final[int]
Buffer_PIXEL_PACK_BUFFER: Final[int]
Buffer_PIXEL_UNPACK_BUFFER: Final[int]
Buffer_READ_ONLY: Final[int]
Buffer_READ_WRITE: Final[int]
Buffer_WRITE_ONLY: Final[int]
LINES: Final[int]
LINE_LOOP: Final[int]
LINE_STRIP: Final[int]
POINTS: Final[int]
POLYGON: Final[int]
QUADS: Final[int]
QUAD_STRIP: Final[int]
TEXTURE2D_DEPTH_COMPONENT: Final[int]
TEXTURE2D_NONE: Final[int]
TEXTURE2D_RGB: Final[int]
TEXTURE2D_RGBA: Final[int]
TRIANGLES: Final[int]
TRIANGLE_FAN: Final[int]
TRIANGLE_STRIP: Final[int]
Texture2D_DEPTH_COMPONENT: Final[int]
Texture2D_NONE: Final[int]
Texture2D_RGB: Final[int]
Texture2D_RGBA: Final[int]
