from typing import Final

CIRCLE: Final[int]
COMEHERE: Final[int]
CROSS: Final[int]
DARK: Final[int]
FEMALE: Final[int]
FIST: Final[int]
FLAT: Final[int]
HUMANEVA_1: Final[int]
HUMANEVA_2: Final[int]
INDEX: Final[int]
LADYBUG: Final[int]
LEFT: Final[int]
LIGHT: Final[int]
MALE: Final[int]
NEG: Final[int]
NONE: Final[int]
PAPER_WITH_CHARACTERS: Final[int]
PAT: Final[int]
POS: Final[int]
RIGHT: Final[int]
RIGHTLEFT: Final[int]
TRIANGLE: Final[int]
TURNAROUND: Final[int]
UPDOWN: Final[int]
WAVE: Final[int]
WHITE_PAPER: Final[int]
WOODEN_BOARD: Final[int]
Z: Final[int]
circle: Final[int]
comehere: Final[int]
cross: Final[int]
dark: Final[int]
female: Final[int]
fist: Final[int]
flat: Final[int]
humaneva_1: Final[int]
humaneva_2: Final[int]
index: Final[int]
light: Final[int]
male: Final[int]
none: Final[int]
paperWithCharacters: Final[int]
pat: Final[int]
rightleft: Final[int]
triangle: Final[int]
turnaround: Final[int]
updown: Final[int]
wave: Final[int]
whitePaper: Final[int]
woodenBoard: Final[int]
z: Final[int]
