import builtins
from typing import Final

class GStreamerPipeline: ...

GSTREAMER_SOURCE_OUTPUT_TYPE_FRAME: Final[int]
GSTREAMER_SOURCE_OUTPUT_TYPE_MAT: Final[int]
GStreamerSource_OutputType_FRAME: Final[int]
GStreamerSource_OutputType_MAT: Final[int]
