from typing import Final

MULTI_CAMERA_CALIBRATION_OMNIDIRECTIONAL: Final[int] = 1
MULTI_CAMERA_CALIBRATION_PINHOLE: Final[int] = 0
MultiCameraCalibration_OMNIDIRECTIONAL: Final[int] = 1
MultiCameraCalibration_PINHOLE: Final[int] = 0
